@@ -791,6 +791,7 @@ g_content_type_get_icon (const char *type)
   themed_icon = g_themed_icon_new_from_names (icon_names, n);
   
   g_free (xdg_mimetype_icon);
+  g_free (xdg_mimetype_generic_icon);
   g_free (mimetype_icon);
   g_free (legacy_mimetype_icon);
   g_free (generic_mimetype_icon);
@@ -932,6 +933,16 @@ g_content_type_guess (const char   *filename,
 	  data &&
 	  looks_like_text (data, data_size))
 	sniffed_mimetype = "text/plain";
+
+      /* For security reasons we don't ever want to sniff desktop files
+       * where we know the filename and it doesn't have a .desktop extension.
+       * This is because desktop files allow executing any application and
+       * we don't want to make it possible to hide them looking like something
+       * else.
+       */
+      if (filename != NULL &&
+          strcmp (sniffed_mimetype, "application/x-desktop") == 0)
+        sniffed_mimetype = "text/plain";
     }
   
   if (n_name_mimetypes == 0)
@@ -998,7 +1009,7 @@ enumerate_mimetypes_subdir (const char *dir,
 	  if (g_str_has_suffix (ent->d_name, ".xml"))
 	    {
 	      mimetype = g_strdup_printf ("%s/%.*s", prefix, (int) strlen (ent->d_name) - 4, ent->d_name);
-	      g_hash_table_insert (mimetypes, mimetype, NULL);
+	      g_hash_table_replace (mimetypes, mimetype, NULL);
 	    }
 	}
       closedir (d);
@@ -1053,7 +1064,7 @@ g_content_types_get_registered (void)
   int i;
   GList *l;
 
-  mimetypes = g_hash_table_new (g_str_hash, g_str_equal);
+  mimetypes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
   enumerate_mimetypes_dir (g_get_user_data_dir (), mimetypes);
   dirs = g_get_system_data_dirs ();
@@ -1064,7 +1075,10 @@ g_content_types_get_registered (void)
   l = NULL;
   g_hash_table_iter_init (&iter, mimetypes);
   while (g_hash_table_iter_next (&iter, &key, NULL))
-    l = g_list_prepend (l, key);
+    {
+      l = g_list_prepend (l, key);
+      g_hash_table_iter_steal (&iter);
+    }
 
   g_hash_table_destroy (mimetypes);
 
