@@ -364,10 +364,9 @@ g_time_val_from_iso8601 (const gchar *iso_date,
       tm.tm_hour = val / 10000;
     }
 
-  time_->tv_sec = mktime_utc (&tm);
   time_->tv_usec = 0;
   
-  if (*iso_date == '.')
+  if (*iso_date == ',' || *iso_date == '.')
     {
       glong mul = 100000;
 
@@ -378,21 +377,30 @@ g_time_val_from_iso8601 (const gchar *iso_date,
         }
     }
     
-  if (*iso_date == '+' || *iso_date == '-')
+  /* Now parse the offset and convert tm to a time_t */
+  if (*iso_date == 'Z')
+    {
+      iso_date++;
+      time_->tv_sec = mktime_utc (&tm);
+    }
+  else if (*iso_date == '+' || *iso_date == '-')
     {
       gint sign = (*iso_date == '+') ? -1 : 1;
       
-      val = 60 * strtoul (iso_date + 1, (char **)&iso_date, 10);
+      val = strtoul (iso_date + 1, (char **)&iso_date, 10);
       
       if (*iso_date == ':')
 	val = 60 * val + strtoul (iso_date + 1, (char **)&iso_date, 10);
       else
         val = 60 * (val / 100) + (val % 100);
 
-      time_->tv_sec += (time_t) (val * sign);
+      time_->tv_sec = mktime_utc (&tm) + (time_t) (60 * val * sign);
+    }
+  else
+    {
+      /* No "Z" or offset, so local time */
+      time_->tv_sec = mktime (&tm);
     }
-  else if (*iso_date++ != 'Z')
-    return FALSE;
 
   while (g_ascii_isspace (*iso_date))
     iso_date++;
@@ -419,19 +427,18 @@ g_time_val_to_iso8601 (GTimeVal *time_)
 #ifdef HAVE_GMTIME_R
   struct tm tm_;
 #endif
+  time_t secs;
   
   g_return_val_if_fail (time_->tv_usec >= 0 && time_->tv_usec < G_USEC_PER_SEC, NULL);
 
-#ifdef _WIN64
-  {
-    time_t secs = time_->tv_sec;
-    tm = gmtime (&secs);
-  }
+ secs = time_->tv_sec;
+#ifdef _WIN32
+ tm = gmtime (&secs);
 #else
 #ifdef HAVE_GMTIME_R
-  tm = gmtime_r (&time_->tv_sec, &tm_);
+  tm = gmtime_r (&secs, &tm_);
 #else
-  tm = gmtime (&time_->tv_sec);
+  tm = gmtime (&secs);
 #endif
 #endif
 
